(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global['polished-mixins'] = {})));
}(this, (function (exports) { 'use strict';

  var cssRegex = /^([+-]?(?:\d+|\d*\.\d+))([a-z]*|%)$/;
  /**
   * Returns a given CSS value and its unit as elements of an array.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   '--dimension': getValueAndUnit('100px')[0]
   *   '--unit': getValueAndUnit('100px')[1]
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   --dimension: ${getValueAndUnit('100px')[0]}
   *   --unit: ${getValueAndUnit('100px')[1]}
   * `
   *
   * // CSS in JS Output
   *
   * element {
   *   '--dimension': 100
   *   '--unit': 'px'
   * }
   */

  function getValueAndUnit(value) {
    if (typeof value !== 'string') return [value, ''];
    var matchedValue = value.match(cssRegex);
    if (matchedValue) return [parseFloat(value), matchedValue[2]];
    return [value, undefined];
  }

  /**
   * Returns a CSS calc formula for linear interpolation of a property between two values. Accepts optional minScreen (defaults to '320px') and maxScreen (defaults to '1200px').
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   fontSize: between('20px', '100px', '400px', '1000px'),
   *   fontSize: between('20px', '100px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   fontSize: ${fontSize: between('20px', '100px', '400px', '1000px')};
   *   fontSize: ${fontSize: between('20px', '100px')}
   * `
   *
   * // CSS as JS Output
   *
   * h1: {
   *   'fontSize': 'calc(-33.33333333333334px + 13.333333333333334vw)',
   *   'fontSize': 'calc(-9.090909090909093px + 9.090909090909092vw)'
   * }
   */

  function between(fromSize, toSize, minScreen, maxScreen) {
    if (minScreen === void 0) {
      minScreen = '320px';
    }

    if (maxScreen === void 0) {
      maxScreen = '1200px';
    }

    var _getValueAndUnit = getValueAndUnit(fromSize),
        unitlessFromSize = _getValueAndUnit[0],
        fromSizeUnit = _getValueAndUnit[1];

    var _getValueAndUnit2 = getValueAndUnit(toSize),
        unitlessToSize = _getValueAndUnit2[0],
        toSizeUnit = _getValueAndUnit2[1];

    var _getValueAndUnit3 = getValueAndUnit(minScreen),
        unitlessMinScreen = _getValueAndUnit3[0],
        minScreenUnit = _getValueAndUnit3[1];

    var _getValueAndUnit4 = getValueAndUnit(maxScreen),
        unitlessMaxScreen = _getValueAndUnit4[0],
        maxScreenUnit = _getValueAndUnit4[1];

    if (typeof unitlessMinScreen !== 'number' || typeof unitlessMaxScreen !== 'number' || !minScreenUnit || !maxScreenUnit || minScreenUnit !== maxScreenUnit) {
      throw new Error('minScreen and maxScreen must be provided as stringified numbers with the same units.');
    }

    if (typeof unitlessFromSize !== 'number' || typeof unitlessToSize !== 'number' || !fromSizeUnit || !toSizeUnit || fromSizeUnit !== toSizeUnit) {
      throw new Error('fromSize and toSize must be provided as stringified numbers with the same units.');
    }

    var slope = (unitlessFromSize - unitlessToSize) / (unitlessMinScreen - unitlessMaxScreen);
    var base = unitlessToSize - slope * unitlessMaxScreen;
    return "calc(" + base.toFixed(2) + fromSizeUnit + " + " + (100 * slope).toFixed(2) + "vw)";
  }

  /**
   * CSS to contain a float (credit to CSSMojo).
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *    ...clearFix(),
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${clearFix()}
   * `
   *
   * // CSS as JS Output
   *
   * '&::after': {
   *   'clear': 'both',
   *   'content': '""',
   *   'display': 'table'
   * }
   */
  function clearFix(parent) {
    var _ref;

    if (parent === void 0) {
      parent = '&';
    }

    var pseudoSelector = parent + "::after";
    return _ref = {}, _ref[pseudoSelector] = {
      clear: 'both',
      content: '""',
      display: 'table'
    }, _ref;
  }

  /**
   * CSS to fully cover an area. Can optionally be passed an offset to act as a "padding".
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...cover()
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${cover()}
   * `
   *
   * // CSS as JS Output
   *
   * div: {
   *   'position': 'absolute',
   *   'top': '0',
   *   'right: '0',
   *   'bottom': '0',
   *   'left: '0'
   * }
   */
  function cover(offset) {
    if (offset === void 0) {
      offset = 0;
    }

    return {
      position: 'absolute',
      top: offset,
      right: offset,
      bottom: offset,
      left: offset
    };
  }

  /**
   * CSS to represent truncated text with an ellipsis.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...ellipsis('250px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${ellipsis('250px')}
   * `
   *
   * // CSS as JS Output
   *
   * div: {
   *   'display': 'inline-block',
   *   'maxWidth': '250px',
   *   'overflow': 'hidden',
   *   'textOverflow': 'ellipsis',
   *   'whiteSpace': 'nowrap',
   *   'wordWrap': 'normal'
   * }
   */
  function ellipsis(width) {
    if (width === void 0) {
      width = '100%';
    }

    return {
      display: 'inline-block',
      maxWidth: width,
      overflow: 'hidden',
      textOverflow: 'ellipsis',
      whiteSpace: 'nowrap',
      wordWrap: 'normal'
    };
  }

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  /**
   * Returns a set of media queries that resizes a property (or set of properties) between a provided fromSize and toSize. Accepts optional minScreen (defaults to '320px') and maxScreen (defaults to '1200px') to constrain the interpolation.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...fluidRange(
   *    {
   *        prop: 'padding',
   *        fromSize: '20px',
   *        toSize: '100px',
   *      },
   *      '400px',
   *      '1000px',
   *    )
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${fluidRange(
   *      {
   *        prop: 'padding',
   *        fromSize: '20px',
   *        toSize: '100px',
   *      },
   *      '400px',
   *      '1000px',
   *    )}
   * `
   *
   * // CSS as JS Output
   *
   * div: {
   *   "@media (min-width: 1000px)": Object {
   *     "padding": "100px",
   *   },
   *   "@media (min-width: 400px)": Object {
   *     "padding": "calc(-33.33333333333334px + 13.333333333333334vw)",
   *   },
   *   "padding": "20px",
   * }
   */
  function fluidRange(cssProp, minScreen, maxScreen) {
    if (minScreen === void 0) {
      minScreen = '320px';
    }

    if (maxScreen === void 0) {
      maxScreen = '1200px';
    }

    if (!Array.isArray(cssProp) && typeof cssProp !== 'object' || cssProp === null) {
      throw new Error('expects either an array of objects or a single object with the properties prop, fromSize, and toSize.');
    }

    if (Array.isArray(cssProp)) {
      var mediaQueries = {};
      var fallbacks = {};

      for (var _iterator = cssProp, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _extends2, _extends3;

        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var obj = _ref;

        if (!obj.prop || !obj.fromSize || !obj.toSize) {
          throw new Error('expects the objects in the first argument array to have the properties `prop`, `fromSize`, and `toSize`.');
        }

        fallbacks[obj.prop] = obj.fromSize;
        mediaQueries["@media (min-width: " + minScreen + ")"] = _extends({}, mediaQueries["@media (min-width: " + minScreen + ")"], (_extends2 = {}, _extends2[obj.prop] = between(obj.fromSize, obj.toSize, minScreen, maxScreen), _extends2));
        mediaQueries["@media (min-width: " + maxScreen + ")"] = _extends({}, mediaQueries["@media (min-width: " + maxScreen + ")"], (_extends3 = {}, _extends3[obj.prop] = obj.toSize, _extends3));
      }

      return _extends({}, fallbacks, mediaQueries);
    } else {
      var _ref2, _ref3, _ref4;

      if (!cssProp.prop || !cssProp.fromSize || !cssProp.toSize) {
        throw new Error('expects the first argument object to have the properties `prop`, `fromSize`, and `toSize`.');
      }

      return _ref4 = {}, _ref4[cssProp.prop] = cssProp.fromSize, _ref4["@media (min-width: " + minScreen + ")"] = (_ref2 = {}, _ref2[cssProp.prop] = between(cssProp.fromSize, cssProp.toSize, minScreen, maxScreen), _ref2), _ref4["@media (min-width: " + maxScreen + ")"] = (_ref3 = {}, _ref3[cssProp.prop] = cssProp.toSize, _ref3), _ref4;
    }
  }

  function generateFileReferences(fontFilePath, fileFormats) {
    var fileFontReferences = fileFormats.map(function (format) {
      return "url(\"" + fontFilePath + "." + format + "\")";
    });
    return fileFontReferences.join(', ');
  }

  function generateLocalReferences(localFonts) {
    var localFontReferences = localFonts.map(function (font) {
      return "local(\"" + font + "\")";
    });
    return localFontReferences.join(', ');
  }

  function generateSources(fontFilePath, localFonts, fileFormats) {
    var fontReferences = [];
    if (localFonts) fontReferences.push(generateLocalReferences(localFonts));

    if (fontFilePath) {
      fontReferences.push(generateFileReferences(fontFilePath, fileFormats));
    }

    return fontReferences.join(', ');
  }
  /**
   * CSS for a @font-face declaration.
   *
   * @example
   * // Styles as object basic usage
   * const styles = {
   *    ...fontFace({
   *      'fontFamily': 'Sans-Pro',
   *      'fontFilePath': 'path/to/file'
   *    })
   * }
   *
   * // styled-components basic usage
   * injectGlobal`${
   *   fontFace({
   *     'fontFamily': 'Sans-Pro',
   *     'fontFilePath': 'path/to/file'
   *   }
   * )}`
   *
   * // CSS as JS Output
   *
   * '@font-face': {
   *   'fontFamily': 'Sans-Pro',
   *   'src': 'url("path/to/file.eot"), url("path/to/file.woff2"), url("path/to/file.woff"), url("path/to/file.ttf"), url("path/to/file.svg")',
   * }
   */


  function fontFace(_ref) {
    var fontFamily = _ref.fontFamily,
        fontFilePath = _ref.fontFilePath,
        fontStretch = _ref.fontStretch,
        fontStyle = _ref.fontStyle,
        fontVariant = _ref.fontVariant,
        fontWeight = _ref.fontWeight,
        _ref$fileFormats = _ref.fileFormats,
        fileFormats = _ref$fileFormats === void 0 ? ['eot', 'woff2', 'woff', 'ttf', 'svg'] : _ref$fileFormats,
        localFonts = _ref.localFonts,
        unicodeRange = _ref.unicodeRange,
        fontDisplay = _ref.fontDisplay,
        fontVariationSettings = _ref.fontVariationSettings,
        fontFeatureSettings = _ref.fontFeatureSettings;
    // Error Handling
    if (!fontFamily) throw new Error('fontFace expects a name of a font-family.');

    if (!fontFilePath && !localFonts) {
      throw new Error('fontFace expects either the path to the font file(s) or a name of a local copy.');
    }

    if (localFonts && !Array.isArray(localFonts)) {
      throw new Error('fontFace expects localFonts to be an array.');
    }

    if (!Array.isArray(fileFormats)) {
      throw new Error('fontFace expects fileFormats to be an array.');
    }

    var fontFaceDeclaration = {
      '@font-face': {
        fontFamily: fontFamily,
        src: generateSources(fontFilePath, localFonts, fileFormats),
        unicodeRange: unicodeRange,
        fontStretch: fontStretch,
        fontStyle: fontStyle,
        fontVariant: fontVariant,
        fontWeight: fontWeight,
        fontDisplay: fontDisplay,
        fontVariationSettings: fontVariationSettings,
        fontFeatureSettings: fontFeatureSettings
      } // Removes undefined fields for cleaner css object.

    };
    return JSON.parse(JSON.stringify(fontFaceDeclaration));
  }

  /**
   * CSS to hide text to show a background image in a SEO-friendly way.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   'backgroundImage': 'url(logo.png)',
   *   ...hideText(),
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   backgroundImage: url(logo.png);
   *   ${hideText()};
   * `
   *
   * // CSS as JS Output
   *
   * 'div': {
   *   'backgroundImage': 'url(logo.png)',
   *   'textIndent': '101%',
   *   'overflow': 'hidden',
   *   'whiteSpace': 'nowrap',
   * }
   */
  function hideText() {
    return {
      textIndent: '101%',
      overflow: 'hidden',
      whiteSpace: 'nowrap'
    };
  }

  /**
   * CSS to hide content visually but remain accessible to screen readers.
   * from [HTML5 Boilerplate](https://github.com/h5bp/html5-boilerplate/blob/9a176f57af1cfe8ec70300da4621fb9b07e5fa31/src/css/main.css#L121)
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...hideVisually(),
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${hideVisually()};
   * `
   *
   * // CSS as JS Output
   *
   * 'div': {
   *   'border': '0',
   *   'clip': 'rect(0 0 0 0)',
   *   'clipPath': 'inset(50%)',
   *   'height': '1px',
   *   'margin': '-1px',
   *   'overflow': 'hidden',
   *   'padding': '0',
   *   'position': 'absolute',
   *   'whiteSpace': 'nowrap',
   *   'width': '1px',
   * }
   */
  function hideVisually() {
    return {
      border: '0',
      clip: 'rect(0 0 0 0)',
      clipPath: 'inset(50%)',
      height: '1px',
      margin: '-1px',
      overflow: 'hidden',
      padding: '0',
      position: 'absolute',
      whiteSpace: 'nowrap',
      width: '1px'
    };
  }

  /**
   * Generates a media query to target HiDPI devices.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *  [hiDPI(1.5)]: {
   *    width: 200px;
   *  }
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${hiDPI(1.5)} {
   *     width: 200px;
   *   }
   * `
   *
   * // CSS as JS Output
   *
   * '@media only screen and (-webkit-min-device-pixel-ratio: 1.5),
   *  only screen and (min--moz-device-pixel-ratio: 1.5),
   *  only screen and (-o-min-device-pixel-ratio: 1.5/1),
   *  only screen and (min-resolution: 144dpi),
   *  only screen and (min-resolution: 1.5dppx)': {
   *   'width': '200px',
   * }
   */
  function hiDPI(ratio) {
    if (ratio === void 0) {
      ratio = 1.3;
    }

    return "\n    @media only screen and (-webkit-min-device-pixel-ratio: " + ratio + "),\n    only screen and (min--moz-device-pixel-ratio: " + ratio + "),\n    only screen and (-o-min-device-pixel-ratio: " + ratio + "/1),\n    only screen and (min-resolution: " + Math.round(ratio * 96) + "dpi),\n    only screen and (min-resolution: " + ratio + "dppx)\n  ";
  }

  /**
   * CSS to normalize abnormalities across browsers (normalize.css v8.0.0 | MIT License | github.com/necolas/normalize.css)
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *    ...normalize(),
   * }
   *
   * // styled-components usage
   * injectGlobal`${normalize()}`
   *
   * // CSS as JS Output
   *
   * html {
   *   lineHeight: 1.15,
   *   textSizeAdjust: 100%,
   * } ...
   */
  function normalize() {
    var _ref;

    return [(_ref = {
      html: {
        lineHeight: '1.15',
        textSizeAdjust: '100%'
      },
      body: {
        margin: '0'
      },
      h1: {
        fontSize: '2em',
        margin: '0.67em 0'
      },
      hr: {
        boxSizing: 'content-box',
        height: '0',
        overflow: 'visible'
      },
      pre: {
        fontFamily: 'monospace, monospace',
        fontSize: '1em'
      },
      a: {
        'background-color': 'transparent'
      },
      'abbr[title]': {
        borderBottom: 'none',
        textDecoration: 'underline'
      }
    }, _ref["b,\n    strong"] = {
      fontWeight: 'bolder'
    }, _ref["code,\n    kbd,\n    samp"] = {
      fontFamily: 'monospace, monospace',
      fontSize: '1em'
    }, _ref.small = {
      fontSize: '80%'
    }, _ref["sub,\n    sup"] = {
      fontSize: '75%',
      lineHeight: '0',
      position: 'relative',
      verticalAlign: 'baseline'
    }, _ref.sub = {
      bottom: '-0.25em'
    }, _ref.sup = {
      top: '-0.5em'
    }, _ref.img = {
      borderStyle: 'none'
    }, _ref["button,\n    input,\n    optgroup,\n    select,\n    textarea"] = {
      fontFamily: 'inherit',
      fontSize: '100%',
      lineHeight: '1.15',
      margin: '0'
    }, _ref["button,\n    input"] = {
      overflow: 'visible'
    }, _ref["button,\n    select"] = {
      textTransform: 'none'
    }, _ref["button,\n    html [type=\"button\"],\n    [type=\"reset\"],\n    [type=\"submit\"]"] = {
      WebkitAppearance: 'button'
    }, _ref["button::-moz-focus-inner,\n    [type=\"button\"]::-moz-focus-inner,\n    [type=\"reset\"]::-moz-focus-inner,\n    [type=\"submit\"]::-moz-focus-inner"] = {
      borderStyle: 'none',
      padding: '0'
    }, _ref["button:-moz-focusring,\n    [type=\"button\"]:-moz-focusring,\n    [type=\"reset\"]:-moz-focusring,\n    [type=\"submit\"]:-moz-focusring"] = {
      outline: '1px dotted ButtonText'
    }, _ref.fieldset = {
      padding: '0.35em 0.625em 0.75em'
    }, _ref.legend = {
      boxSizing: 'border-box',
      color: 'inherit',
      display: 'table',
      maxWidth: '100%',
      padding: '0',
      whiteSpace: 'normal'
    }, _ref.progress = {
      verticalAlign: 'baseline'
    }, _ref.textarea = {
      overflow: 'auto'
    }, _ref["[type=\"checkbox\"],\n    [type=\"radio\"]"] = {
      boxSizing: 'border-box',
      padding: '0'
    }, _ref["[type=\"number\"]::-webkit-inner-spin-button,\n    [type=\"number\"]::-webkit-outer-spin-button"] = {
      height: 'auto'
    }, _ref['[type="search"]'] = {
      WebkitAppearance: 'textfield',
      outlineOffset: '-2px'
    }, _ref['[type="search"]::-webkit-search-decoration'] = {
      WebkitAppearance: 'none'
    }, _ref['::-webkit-file-upload-button'] = {
      WebkitAppearance: 'button',
      font: 'inherit'
    }, _ref.details = {
      display: 'block'
    }, _ref.summary = {
      display: 'list-item'
    }, _ref.template = {
      display: 'none'
    }, _ref['[hidden]'] = {
      display: 'none'
    }, _ref), {
      'abbr[title]': {
        textDecoration: 'underline dotted'
      }
    }];
  }

  function _taggedTemplateLiteralLoose(strings, raw) {
    if (!raw) {
      raw = strings.slice(0);
    }

    strings.raw = raw;
    return strings;
  }

  function _templateObject() {
    var data = _taggedTemplateLiteralLoose(["radial-gradient(", "", "", "", ")"]);

    _templateObject = function _templateObject() {
      return data;
    };

    return data;
  }

  function parseFallback(colorStops) {
    return colorStops[0].split(' ')[0];
  }

  function constructGradientValue(literals) {
    var template = '';

    for (var i = 0; i < literals.length; i += 1) {
      template += literals[i]; // Adds leading coma if properties preceed color-stops

      if (i === 3 && (i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]) && ((arguments.length <= 1 ? undefined : arguments[1]) || (arguments.length <= 2 ? undefined : arguments[2]) || (arguments.length <= 3 ? undefined : arguments[3]))) {
        template = template.slice(0, -1);
        template += ", " + (i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]); // No trailing space if color-stops is the only param provided
      } else if (i === 3 && (i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]) && !(arguments.length <= 1 ? undefined : arguments[1]) && !(arguments.length <= 2 ? undefined : arguments[2]) && !(arguments.length <= 3 ? undefined : arguments[3])) {
        template += "" + (i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]); // Only adds substitution if it is defined
      } else if (i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]) {
        template += (i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1]) + " ";
      }
    }

    return template.trim();
  }
  /**
   * CSS for declaring a radial gradient, including a fallback background-color. The fallback is either the first color-stop or an explicitly passed fallback color.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...radialGradient({
   *     colorStops: ['#00FFFF 0%', 'rgba(0, 0, 255, 0) 50%', '#0000FF 95%'],
   *     extent: 'farthest-corner at 45px 45px',
   *     position: 'center',
   *     shape: 'ellipse',
   *   })
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${radialGradient({
   *     colorStops: ['#00FFFF 0%', 'rgba(0, 0, 255, 0) 50%', '#0000FF 95%'],
   *     extent: 'farthest-corner at 45px 45px',
   *     position: 'center',
   *     shape: 'ellipse',
   *   })}
   *`
   *
   * // CSS as JS Output
   *
   * div: {
   *   'backgroundColor': '#00FFFF',
   *   'backgroundImage': 'radial-gradient(center ellipse farthest-corner at 45px 45px, #00FFFF 0%, rgba(0, 0, 255, 0) 50%, #0000FF 95%)',
   * }
   */


  function radialGradient(_ref) {
    var colorStops = _ref.colorStops,
        extent = _ref.extent,
        fallback = _ref.fallback,
        position = _ref.position,
        shape = _ref.shape;

    if (!colorStops || colorStops.length < 2) {
      throw new Error('radialGradient requries at least 2 color-stops to properly render.');
    }

    return {
      backgroundColor: fallback || parseFallback(colorStops),
      backgroundImage: constructGradientValue(_templateObject(), position, shape, extent, colorStops.join(', '))
    };
  }

  /**
   * A helper to generate a retina background image and non-retina
   * background image. The retina background image will output to a HiDPI media query. The mixin uses
   * a _2x.png filename suffix by default.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *  ...retinaImage('my-img')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${retinaImage('my-img')}
   * `
   *
   * // CSS as JS Output
   * div {
   *   backgroundImage: 'url(my-img.png)',
   *   '@media only screen and (-webkit-min-device-pixel-ratio: 1.3),
   *    only screen and (min--moz-device-pixel-ratio: 1.3),
   *    only screen and (-o-min-device-pixel-ratio: 1.3/1),
   *    only screen and (min-resolution: 144dpi),
   *    only screen and (min-resolution: 1.5dppx)': {
   *     backgroundImage: 'url(my-img_2x.png)',
   *   }
   * }
   */
  function retinaImage(filename, backgroundSize, extension, retinaFilename, retinaSuffix) {
    var _ref;

    if (extension === void 0) {
      extension = 'png';
    }

    if (retinaSuffix === void 0) {
      retinaSuffix = '_2x';
    }

    if (!filename) {
      throw new Error('Please supply a filename to retinaImage() as the first argument.');
    } // Replace the dot at the beginning of the passed extension if one exists


    var ext = extension.replace(/^\./, '');
    var rFilename = retinaFilename ? retinaFilename + "." + ext : "" + filename + retinaSuffix + "." + ext;
    return _ref = {
      backgroundImage: "url(" + filename + "." + ext + ")"
    }, _ref[hiDPI()] = _extends({
      backgroundImage: "url(" + rFilename + ")"
    }, backgroundSize ? {
      backgroundSize: backgroundSize
    } : {}), _ref;
  }

  /* eslint-disable key-spacing */
  var functionsMap = {
    easeInBack: 'cubic-bezier(0.600, -0.280, 0.735, 0.045)',
    easeInCirc: 'cubic-bezier(0.600,  0.040, 0.980, 0.335)',
    easeInCubic: 'cubic-bezier(0.550,  0.055, 0.675, 0.190)',
    easeInExpo: 'cubic-bezier(0.950,  0.050, 0.795, 0.035)',
    easeInQuad: 'cubic-bezier(0.550,  0.085, 0.680, 0.530)',
    easeInQuart: 'cubic-bezier(0.895,  0.030, 0.685, 0.220)',
    easeInQuint: 'cubic-bezier(0.755,  0.050, 0.855, 0.060)',
    easeInSine: 'cubic-bezier(0.470,  0.000, 0.745, 0.715)',
    easeOutBack: 'cubic-bezier(0.175,  0.885, 0.320, 1.275)',
    easeOutCubic: 'cubic-bezier(0.215,  0.610, 0.355, 1.000)',
    easeOutCirc: 'cubic-bezier(0.075,  0.820, 0.165, 1.000)',
    easeOutExpo: 'cubic-bezier(0.190,  1.000, 0.220, 1.000)',
    easeOutQuad: 'cubic-bezier(0.250,  0.460, 0.450, 0.940)',
    easeOutQuart: 'cubic-bezier(0.165,  0.840, 0.440, 1.000)',
    easeOutQuint: 'cubic-bezier(0.230,  1.000, 0.320, 1.000)',
    easeOutSine: 'cubic-bezier(0.390,  0.575, 0.565, 1.000)',
    easeInOutBack: 'cubic-bezier(0.680, -0.550, 0.265, 1.550)',
    easeInOutCirc: 'cubic-bezier(0.785,  0.135, 0.150, 0.860)',
    easeInOutCubic: 'cubic-bezier(0.645,  0.045, 0.355, 1.000)',
    easeInOutExpo: 'cubic-bezier(1.000,  0.000, 0.000, 1.000)',
    easeInOutQuad: 'cubic-bezier(0.455,  0.030, 0.515, 0.955)',
    easeInOutQuart: 'cubic-bezier(0.770,  0.000, 0.175, 1.000)',
    easeInOutQuint: 'cubic-bezier(0.860,  0.000, 0.070, 1.000)',
    easeInOutSine: 'cubic-bezier(0.445,  0.050, 0.550, 0.950)'
    /* eslint-enable key-spacing */

  };

  function getTimingFunction(functionName) {
    return functionsMap[functionName];
  }
  /**
   * String to represent common easing functions as demonstrated here: (github.com/jaukia/easie).
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   'transitionTimingFunction': timingFunctions('easeInQuad')
   * }
   *
   * // styled-components usage
   *  const div = styled.div`
   *   transitionTimingFunction: ${timingFunctions('easeInQuad')};
   * `
   *
   * // CSS as JS Output
   *
   * 'div': {
   *   'transitionTimingFunction': 'cubic-bezier(0.550,  0.085, 0.680, 0.530)',
   * }
   */


  function timingFunctions(timingFunction) {
    return getTimingFunction(timingFunction);
  }

  // @private
  function capitalizeString(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
  }

  var positionMap = ['Top', 'Right', 'Bottom', 'Left'];

  function generateProperty(property, position) {
    if (!property) return position.toLowerCase();
    var splitProperty = property.split('-');

    if (splitProperty.length > 1) {
      splitProperty.splice(1, 0, position);
      return splitProperty.reduce(function (acc, val) {
        return "" + acc + capitalizeString(val);
      });
    }

    var joinedProperty = property.replace(/([a-z])([A-Z])/g, "$1" + position + "$2");
    return property === joinedProperty ? "" + property + position : joinedProperty;
  }

  function generateStyles(property, valuesWithDefaults) {
    var styles = {};

    for (var i = 0; i < valuesWithDefaults.length; i += 1) {
      if (valuesWithDefaults[i] || valuesWithDefaults[i] === 0) {
        styles[generateProperty(property, positionMap[i])] = valuesWithDefaults[i];
      }
    }

    return styles;
  }
  /**
   * Enables shorthand for direction-based properties. It accepts a property (hyphenated or camelCased) and up to four values that map to top, right, bottom, and left, respectively. You can optionally pass an empty string to get only the directional values as properties. You can also optionally pass a null argument for a directional value to ignore it.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...directionalProperty('padding', '12px', '24px', '36px', '48px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${directionalProperty('padding', '12px', '24px', '36px', '48px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'paddingTop': '12px',
   *   'paddingRight': '24px',
   *   'paddingBottom': '36px',
   *   'paddingLeft': '48px'
   * }
   */


  function directionalProperty(property) {
    for (var _len = arguments.length, values = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      values[_key - 1] = arguments[_key];
    }

    //  prettier-ignore
    var firstValue = values[0],
        _values$ = values[1],
        secondValue = _values$ === void 0 ? firstValue : _values$,
        _values$2 = values[2],
        thirdValue = _values$2 === void 0 ? firstValue : _values$2,
        _values$3 = values[3],
        fourthValue = _values$3 === void 0 ? secondValue : _values$3;
    var valuesWithDefaults = [firstValue, secondValue, thirdValue, fourthValue];
    return generateStyles(property, valuesWithDefaults);
  }

  /**
   * Shorthand that accepts up to four values, including null to skip a value, and maps them to their respective directions.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...borderColor('red', 'green', 'blue', 'yellow')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${borderColor('red', 'green', 'blue', 'yellow')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'borderTopColor': 'red',
   *   'borderRightColor': 'green',
   *   'borderBottomColor': 'blue',
   *   'borderLeftColor': 'yellow'
   * }
   */
  function borderColor() {
    for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }

    return directionalProperty.apply(void 0, ['borderColor'].concat(values));
  }

  var getBorderWidth = function getBorderWidth(pointingDirection, height, width) {
    switch (pointingDirection) {
      case 'top':
        return "0 " + width[0] / 2 + width[1] + " " + height[0] + height[1] + " " + width[0] / 2 + width[1];

      case 'left':
        return "" + height[0] / 2 + height[1] + " " + width[0] + width[1] + " " + height[0] / 2 + height[1] + " 0";

      case 'bottom':
        return "" + height[0] + height[1] + " " + width[0] / 2 + width[1] + " 0 " + width[0] / 2 + width[1];

      case 'right':
        return "" + height[0] / 2 + height[1] + " 0 " + height[0] / 2 + height[1] + " " + width[0] + width[1];

      default:
        throw new Error("Passed invalid argument to triangle, please pass correct pointingDirection e.g. 'right'.");
    }
  }; // needed for border-color


  var reverseDirection = ['bottom', 'left', 'top', 'right'];
  /**
   * CSS to represent triangle with any pointing direction with an optional background color. Accepts number or px values for height and width.
   *
   * @example
   * // Styles as object usage
   *
   * const styles = {
   *   ...triangle({ pointingDirection: 'right', width: '100px', height: '100px', foregroundColor: 'red' })
   * }
   *
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${triangle({ pointingDirection: 'right', width: '100px', height: '100px', foregroundColor: 'red' })}
   *
   *
   * // CSS as JS Output
   *
   * div: {
   *  'borderColor': 'transparent',
   *  'borderLeftColor': 'red !important',
   *  'borderStyle': 'solid',
   *  'borderWidth': '50px 0 50px 100px',
   *  'height': '0',
   *  'width': '0',
   * }
   */

  function triangle(_ref) {
    var pointingDirection = _ref.pointingDirection,
        height = _ref.height,
        width = _ref.width,
        foregroundColor = _ref.foregroundColor,
        _ref$backgroundColor = _ref.backgroundColor,
        backgroundColor = _ref$backgroundColor === void 0 ? 'transparent' : _ref$backgroundColor;
    var widthAndUnit = [parseFloat(width), String(width).replace(/\d+/g, '') || 'px'];
    var heightAndUnit = [parseFloat(height), String(height).replace(/\d+/g, '') || 'px'];

    if (isNaN(heightAndUnit[0]) || isNaN(widthAndUnit[0])) {
      throw new Error('Passed an invalid value to `height` or `width`. Please provide a pixel based unit');
    }

    var reverseDirectionIndex = reverseDirection.indexOf(pointingDirection);
    return _extends({
      width: '0',
      height: '0',
      borderWidth: getBorderWidth(pointingDirection, heightAndUnit, widthAndUnit),
      borderStyle: 'solid'
    }, borderColor.apply(void 0, Array.from({
      length: 4
    }).map(function (_, index) {
      return index === reverseDirectionIndex ? foregroundColor : backgroundColor;
    })));
  }

  /**
   * Provides an easy way to change the `wordWrap` property.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...wordWrap('break-word')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${wordWrap('break-word')}
   * `
   *
   * // CSS as JS Output
   *
   * const styles = {
   *   overflowWrap: 'break-word',
   *   wordWrap: 'break-word',
   *   wordBreak: 'break-all',
   * }
   */
  function wordWrap(wrap) {
    if (wrap === void 0) {
      wrap = 'break-word';
    }

    var wordBreak = wrap === 'break-word' ? 'break-all' : wrap;
    return {
      overflowWrap: wrap,
      wordWrap: wrap,
      wordBreak: wordBreak
    };
  }

  exports.between = between;
  exports.clearFix = clearFix;
  exports.cover = cover;
  exports.ellipsis = ellipsis;
  exports.fluidRange = fluidRange;
  exports.fontFace = fontFace;
  exports.hideText = hideText;
  exports.hideVisually = hideVisually;
  exports.hiDPI = hiDPI;
  exports.normalize = normalize;
  exports.radialGradient = radialGradient;
  exports.retinaImage = retinaImage;
  exports.timingFunctions = timingFunctions;
  exports.triangle = triangle;
  exports.wordWrap = wordWrap;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
