(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global['polished-shorthands'] = {})));
}(this, (function (exports) { 'use strict';

  /**
   * Shorthand for easily setting the animation property. Allows either multiple arrays with animations
   * or a single animation spread over the arguments.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...animation(['rotate', '1s', 'ease-in-out'], ['colorchange', '2s'])
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${animation(['rotate', '1s', 'ease-in-out'], ['colorchange', '2s'])}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'animation': 'rotate 1s ease-in-out, colorchange 2s'
   * }
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...animation('rotate', '1s', 'ease-in-out')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${animation('rotate', '1s', 'ease-in-out')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'animation': 'rotate 1s ease-in-out'
   * }
   */
  function animation() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    // Allow single or multiple animations passed
    var multiMode = Array.isArray(args[0]);

    if (!multiMode && args.length > 8) {
      throw new Error('The animation shorthand only takes 8 arguments. See the specification for more information: http://mdn.io/animation');
    }

    var code = args.map(function (arg) {
      if (multiMode && !Array.isArray(arg) || !multiMode && Array.isArray(arg)) {
        throw new Error("To pass multiple animations please supply them in arrays, e.g. animation(['rotate', '2s'], ['move', '1s'])\nTo pass a single animation please supply them in simple values, e.g. animation('rotate', '2s')");
      }

      if (Array.isArray(arg) && arg.length > 8) {
        throw new Error('The animation shorthand arrays can only have 8 elements. See the specification for more information: http://mdn.io/animation');
      }

      return Array.isArray(arg) ? arg.join(' ') : arg;
    }).join(', ');
    return {
      animation: code
    };
  }

  /**
   * Shorthand that accepts any number of backgroundImage values as parameters for creating a single background statement.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...backgroundImages('url("/image/background.jpg")', 'linear-gradient(red, green)')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${backgroundImages('url("/image/background.jpg")', 'linear-gradient(red, green)')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'backgroundImage': 'url("/image/background.jpg"), linear-gradient(red, green)'
   * }
   */
  function backgroundImages() {
    for (var _len = arguments.length, properties = new Array(_len), _key = 0; _key < _len; _key++) {
      properties[_key] = arguments[_key];
    }

    return {
      backgroundImage: properties.join(', ')
    };
  }

  /**
   * Shorthand that accepts any number of background values as parameters for creating a single background statement.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...backgrounds('url("/image/background.jpg")', 'linear-gradient(red, green)', 'center no-repeat')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${backgrounds('url("/image/background.jpg")', 'linear-gradient(red, green)', 'center no-repeat')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'background': 'url("/image/background.jpg"), linear-gradient(red, green), center no-repeat'
   * }
   */
  function backgrounds() {
    for (var _len = arguments.length, properties = new Array(_len), _key = 0; _key < _len; _key++) {
      properties[_key] = arguments[_key];
    }

    return {
      background: properties.join(', ')
    };
  }

  // @private
  function capitalizeString(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
  }

  var sideMap = ['top', 'right', 'bottom', 'left'];
  /**
   * Shorthand for the border property that splits out individual properties for use with tools like Fela and Styletron. A side keyword can optionally be passed to target only one side's border properties.
   *
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...border('1px', 'solid', 'red')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${border('1px', 'solid', 'red')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'borderColor': 'red',
   *   'borderStyle': 'solid',
   *   'borderWidth': `1px`,
   * }
   *
   * // Styles as object usage
   * const styles = {
   *   ...border('top', '1px', 'solid', 'red')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${border('top', '1px', 'solid', 'red')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'borderTopColor': 'red',
   *   'borderTopStyle': 'solid',
   *   'borderTopWidth': `1px`,
   * }
   */

  function border(sideKeyword) {
    for (var _len = arguments.length, values = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      values[_key - 1] = arguments[_key];
    }

    if (typeof sideKeyword === 'string' && sideMap.indexOf(sideKeyword) >= 0) {
      var _ref;

      return _ref = {}, _ref["border" + capitalizeString(sideKeyword) + "Width"] = values[0], _ref["border" + capitalizeString(sideKeyword) + "Style"] = values[1], _ref["border" + capitalizeString(sideKeyword) + "Color"] = values[2], _ref;
    } else {
      values.unshift(sideKeyword);
      return {
        borderWidth: values[0],
        borderStyle: values[1],
        borderColor: values[2]
      };
    }
  }

  var positionMap = ['Top', 'Right', 'Bottom', 'Left'];

  function generateProperty(property, position) {
    if (!property) return position.toLowerCase();
    var splitProperty = property.split('-');

    if (splitProperty.length > 1) {
      splitProperty.splice(1, 0, position);
      return splitProperty.reduce(function (acc, val) {
        return "" + acc + capitalizeString(val);
      });
    }

    var joinedProperty = property.replace(/([a-z])([A-Z])/g, "$1" + position + "$2");
    return property === joinedProperty ? "" + property + position : joinedProperty;
  }

  function generateStyles(property, valuesWithDefaults) {
    var styles = {};

    for (var i = 0; i < valuesWithDefaults.length; i += 1) {
      if (valuesWithDefaults[i] || valuesWithDefaults[i] === 0) {
        styles[generateProperty(property, positionMap[i])] = valuesWithDefaults[i];
      }
    }

    return styles;
  }
  /**
   * Enables shorthand for direction-based properties. It accepts a property (hyphenated or camelCased) and up to four values that map to top, right, bottom, and left, respectively. You can optionally pass an empty string to get only the directional values as properties. You can also optionally pass a null argument for a directional value to ignore it.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...directionalProperty('padding', '12px', '24px', '36px', '48px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${directionalProperty('padding', '12px', '24px', '36px', '48px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'paddingTop': '12px',
   *   'paddingRight': '24px',
   *   'paddingBottom': '36px',
   *   'paddingLeft': '48px'
   * }
   */


  function directionalProperty(property) {
    for (var _len = arguments.length, values = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      values[_key - 1] = arguments[_key];
    }

    //  prettier-ignore
    var firstValue = values[0],
        _values$ = values[1],
        secondValue = _values$ === void 0 ? firstValue : _values$,
        _values$2 = values[2],
        thirdValue = _values$2 === void 0 ? firstValue : _values$2,
        _values$3 = values[3],
        fourthValue = _values$3 === void 0 ? secondValue : _values$3;
    var valuesWithDefaults = [firstValue, secondValue, thirdValue, fourthValue];
    return generateStyles(property, valuesWithDefaults);
  }

  /**
   * Shorthand that accepts up to four values, including null to skip a value, and maps them to their respective directions.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...borderColor('red', 'green', 'blue', 'yellow')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${borderColor('red', 'green', 'blue', 'yellow')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'borderTopColor': 'red',
   *   'borderRightColor': 'green',
   *   'borderBottomColor': 'blue',
   *   'borderLeftColor': 'yellow'
   * }
   */
  function borderColor() {
    for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }

    return directionalProperty.apply(void 0, ['borderColor'].concat(values));
  }

  /**
   * Shorthand that accepts a value for side and a value for radius and applies the radius value to both corners of the side.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...borderRadius('top', '5px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${borderRadius('top', '5px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'borderTopRightRadius': '5px',
   *   'borderTopLeftRadius': '5px',
   * }
   */
  function borderRadius(side, radius) {
    var uppercaseSide = capitalizeString(side);

    if (!radius && radius !== 0) {
      throw new Error('borderRadius expects a radius value as a string or number as the second argument.');
    }

    if (uppercaseSide === 'Top' || uppercaseSide === 'Bottom') {
      var _ref;

      return _ref = {}, _ref["border" + uppercaseSide + "RightRadius"] = radius, _ref["border" + uppercaseSide + "LeftRadius"] = radius, _ref;
    }

    if (uppercaseSide === 'Left' || uppercaseSide === 'Right') {
      var _ref2;

      return _ref2 = {}, _ref2["borderTop" + uppercaseSide + "Radius"] = radius, _ref2["borderBottom" + uppercaseSide + "Radius"] = radius, _ref2;
    }

    throw new Error('borderRadius expects one of "top", "bottom", "left" or "right" as the first argument.');
  }

  /**
   * Shorthand that accepts up to four values, including null to skip a value, and maps them to their respective directions.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...borderStyle('solid', 'dashed', 'dotted', 'double')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${borderStyle('solid', 'dashed', 'dotted', 'double')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'borderTopStyle': 'solid',
   *   'borderRightStyle': 'dashed',
   *   'borderBottomStyle': 'dotted',
   *   'borderLeftStyle': 'double'
   * }
   */
  function borderStyle() {
    for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }

    return directionalProperty.apply(void 0, ['borderStyle'].concat(values));
  }

  /**
   * Shorthand that accepts up to four values, including null to skip a value, and maps them to their respective directions.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...borderWidth('12px', '24px', '36px', '48px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${borderWidth('12px', '24px', '36px', '48px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'borderTopWidth': '12px',
   *   'borderRightWidth': '24px',
   *   'borderBottomWidth': '36px',
   *   'borderLeftWidth': '48px'
   * }
   */
  function borderWidth() {
    for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }

    return directionalProperty.apply(void 0, ['borderWidth'].concat(values));
  }

  function generateSelectors(template, state) {
    var stateSuffix = state ? ":" + state : '';
    return template(stateSuffix);
  }
  /**
   * Function helper that adds an array of states to a template of selectors. Used in textInputs and buttons.
   * @private
   */


  function statefulSelectors(states, template, stateMap) {
    if (!template) throw new Error('You must provide a template to this method.');
    if (states.length === 0) return generateSelectors(template, null);
    var selectors = [];

    for (var i = 0; i < states.length; i += 1) {
      if (stateMap && stateMap.indexOf(states[i]) < 0) {
        throw new Error('You passed an unsupported selector state to this method.');
      }

      selectors.push(generateSelectors(template, states[i]));
    }

    selectors = selectors.join(',');
    return selectors;
  }

  var stateMap = [undefined, null, 'active', 'focus', 'hover'];

  function template(state) {
    return "button" + state + ",\n  input[type=\"button\"]" + state + ",\n  input[type=\"reset\"]" + state + ",\n  input[type=\"submit\"]" + state;
  }
  /**
   * Populates selectors that target all buttons. You can pass optional states to append to the selectors.
   * @example
   * // Styles as object usage
   * const styles = {
   *   [buttons('active')]: {
   *     'border': 'none'
   *   }
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   > ${buttons('active')} {
   *     border: none;
   *   }
   * `
   *
   * // CSS in JS Output
   *
   *  'button:active,
   *  'input[type="button"]:active,
   *  'input[type=\"reset\"]:active,
   *  'input[type=\"submit\"]:active: {
   *   'border': 'none'
   * }
   */


  function buttons() {
    for (var _len = arguments.length, states = new Array(_len), _key = 0; _key < _len; _key++) {
      states[_key] = arguments[_key];
    }

    return statefulSelectors(states, template, stateMap);
  }

  /**
   * Shorthand that accepts up to four values, including null to skip a value, and maps them to their respective directions.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...margin('12px', '24px', '36px', '48px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${margin('12px', '24px', '36px', '48px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'marginTop': '12px',
   *   'marginRight': '24px',
   *   'marginBottom': '36px',
   *   'marginLeft': '48px'
   * }
   */
  function margin() {
    for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }

    return directionalProperty.apply(void 0, ['margin'].concat(values));
  }

  /**
   * Shorthand that accepts up to four values, including null to skip a value, and maps them to their respective directions.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...padding('12px', '24px', '36px', '48px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${padding('12px', '24px', '36px', '48px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'paddingTop': '12px',
   *   'paddingRight': '24px',
   *   'paddingBottom': '36px',
   *   'paddingLeft': '48px'
   * }
   */
  function padding() {
    for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
      values[_key] = arguments[_key];
    }

    return directionalProperty.apply(void 0, ['padding'].concat(values));
  }

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  var positionMap$1 = ['absolute', 'fixed', 'relative', 'static', 'sticky'];
  /**
   * Shorthand accepts up to five values, including null to skip a value, and maps them to their respective directions. The first value can optionally be a position keyword.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...position('12px', '24px', '36px', '48px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${position('12px', '24px', '36px', '48px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'top': '12px',
   *   'right': '24px',
   *   'bottom': '36px',
   *   'left': '48px'
   * }
   *
   * // Styles as object usage
   * const styles = {
   *   ...position('absolute', '12px', '24px', '36px', '48px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${position('absolute', '12px', '24px', '36px', '48px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'position': 'absolute',
   *   'top': '12px',
   *   'right': '24px',
   *   'bottom': '36px',
   *   'left': '48px'
   * }
   */

  function position(positionKeyword) {
    for (var _len = arguments.length, values = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      values[_key - 1] = arguments[_key];
    }

    if (positionMap$1.indexOf(positionKeyword) >= 0) {
      return _extends({
        position: positionKeyword
      }, directionalProperty.apply(void 0, [''].concat(values)));
    } else {
      var firstValue = positionKeyword; // in this case position is actually the first value

      return directionalProperty.apply(void 0, ['', firstValue].concat(values));
    }
  }

  /**
   * Shorthand to set the height and width properties in a single statement.
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...size('300px', '250px')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${size('300px', '250px')}
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'height': '300px',
   *   'width': '250px',
   * }
   */
  function size(height, width) {
    if (width === void 0) {
      width = height;
    }

    return {
      height: height,
      width: width
    };
  }

  var stateMap$1 = [undefined, null, 'active', 'focus', 'hover'];

  function template$1(state) {
    return "input[type=\"color\"]" + state + ",\n    input[type=\"date\"]" + state + ",\n    input[type=\"datetime\"]" + state + ",\n    input[type=\"datetime-local\"]" + state + ",\n    input[type=\"email\"]" + state + ",\n    input[type=\"month\"]" + state + ",\n    input[type=\"number\"]" + state + ",\n    input[type=\"password\"]" + state + ",\n    input[type=\"search\"]" + state + ",\n    input[type=\"tel\"]" + state + ",\n    input[type=\"text\"]" + state + ",\n    input[type=\"time\"]" + state + ",\n    input[type=\"url\"]" + state + ",\n    input[type=\"week\"]" + state + ",\n    input:not([type])" + state + ",\n    textarea" + state;
  }
  /**
   * Populates selectors that target all text inputs. You can pass optional states to append to the selectors.
   * @example
   * // Styles as object usage
   * const styles = {
   *   [textInputs('active')]: {
   *     'border': 'none'
   *   }
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   > ${textInputs('active')} {
   *     border: none;
   *   }
   * `
   *
   * // CSS in JS Output
   *
   *  'input[type="color"]:active,
   *  input[type="date"]:active,
   *  input[type="datetime"]:active,
   *  input[type="datetime-local"]:active,
   *  input[type="email"]:active,
   *  input[type="month"]:active,
   *  input[type="number"]:active,
   *  input[type="password"]:active,
   *  input[type="search"]:active,
   *  input[type="tel"]:active,
   *  input[type="text"]:active,
   *  input[type="time"]:active,
   *  input[type="url"]:active,
   *  input[type="week"]:active,
   *  input:not([type]):active,
   *  textarea:active': {
   *   'border': 'none'
   * }
   */


  function textInputs() {
    for (var _len = arguments.length, states = new Array(_len), _key = 0; _key < _len; _key++) {
      states[_key] = arguments[_key];
    }

    return statefulSelectors(states, template$1, stateMap$1);
  }

  /**
   * Accepts any number of transition values as parameters for creating a single transition statement. You may also pass an array of properties as the first parameter that you would like to apply the same tranisition values to (second parameter).
   * @example
   * // Styles as object usage
   * const styles = {
   *   ...transitions('opacity 1.0s ease-in 0s', 'width 2.0s ease-in 2s'),
   *   ...transitions(['color', 'background-color'], '2.0s ease-in 2s')
   * }
   *
   * // styled-components usage
   * const div = styled.div`
   *   ${transitions('opacity 1.0s ease-in 0s', 'width 2.0s ease-in 2s')};
   *   ${transitions(['color', 'background-color'], '2.0s ease-in 2s'),};
   * `
   *
   * // CSS as JS Output
   *
   * div {
   *   'transition': 'opacity 1.0s ease-in 0s, width 2.0s ease-in 2s'
   *   'transition': 'color 2.0s ease-in 2s, background-color 2.0s ease-in 2s',
   * }
   */
  function transitions() {
    for (var _len = arguments.length, properties = new Array(_len), _key = 0; _key < _len; _key++) {
      properties[_key] = arguments[_key];
    }

    if (Array.isArray(properties[0]) && properties.length === 2) {
      var value = properties[1];

      if (typeof value !== 'string') {
        throw new Error('Property must be a string value.');
      }

      var transitionsString = properties[0].map(function (property) {
        return property + " " + value;
      }).join(', ');
      return {
        transition: transitionsString
      };
    } else {
      return {
        transition: properties.join(', ')
      };
    }
  }

  exports.animation = animation;
  exports.backgroundImages = backgroundImages;
  exports.backgrounds = backgrounds;
  exports.border = border;
  exports.borderColor = borderColor;
  exports.borderRadius = borderRadius;
  exports.borderStyle = borderStyle;
  exports.borderWidth = borderWidth;
  exports.buttons = buttons;
  exports.margin = margin;
  exports.padding = padding;
  exports.position = position;
  exports.size = size;
  exports.textInputs = textInputs;
  exports.transitions = transitions;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
